@@ -18,17 +18,18 @@
 GIT_URL="git://git.kernel.org/pub/scm/linux/kernel/git/next/linux-next.git"
 GIT_COMPAT_URL="git://git.kernel.org/pub/scm/linux/kernel/git/mcgrof/compat.git"
 
-INCLUDE_NET_BT="hci_core.h l2cap.h bluetooth.h rfcomm.h hci.h"
+INCLUDE_NET_BT="hci_core.h l2cap.h bluetooth.h rfcomm.h hci.h mgmt.h smp.h"
 NET_BT_DIRS="bluetooth bluetooth/bnep bluetooth/cmtp bluetooth/rfcomm bluetooth/hidp"
 
 INCLUDE_LINUX="ieee80211.h nl80211.h"
 INCLUDE_LINUX="$INCLUDE_LINUX pci_ids.h eeprom_93cx6.h"
 INCLUDE_LINUX="$INCLUDE_LINUX ath9k_platform.h"
+INCLUDE_LINUX="$INCLUDE_LINUX wl12xx.h"
 
 # For rndis_wext
 INCLUDE_LINUX_USB="usbnet.h rndis_host.h"
 
-INCLUDE_LINUX_SPI="wl12xx.h libertas_spi.h"
+INCLUDE_LINUX_SPI="libertas_spi.h"
 
 # The good new yummy stuff
 INCLUDE_NET="cfg80211.h ieee80211_radiotap.h"
@@ -190,7 +191,7 @@ if [ -z $GIT_TREE ]; then
 		echo "  export GIT_TREE=/home/$USER/linux-next/"
 		echo
 		echo "If you do not have one you can clone the repository:"
-		echo "  git-clone $GIT_URL"
+		echo "  git clone $GIT_URL"
 		exit 1
 	fi
 else
@@ -206,7 +207,7 @@ if [ -z $GIT_COMPAT_TREE ]; then
 		echo "  export GIT_COMPAT_TREE=/home/$USER/compat/"
 		echo
 		echo "If you do not have one you can clone the repository:"
-		echo "  git-clone $GIT_COMPAT_URL"
+		echo "  git clone $GIT_COMPAT_URL"
 		exit 1
 	fi
 else
@@ -215,23 +216,39 @@ fi
 
 # Drivers that have their own directory
 DRIVERS="drivers/net/wireless/ath"
-DRIVERS="$DRIVERS drivers/net/wireless/ath/ar9170"
+DRIVERS="$DRIVERS drivers/net/wireless/ath/carl9170"
 DRIVERS="$DRIVERS drivers/net/wireless/ath/ath5k"
 DRIVERS="$DRIVERS drivers/net/wireless/ath/ath9k"
 DRIVERS="$DRIVERS drivers/ssb"
+DRIVERS="$DRIVERS drivers/bcma"
 DRIVERS="$DRIVERS drivers/net/wireless/b43"
 DRIVERS="$DRIVERS drivers/net/wireless/b43legacy"
+DRIVERS="$DRIVERS drivers/net/wireless/iwlegacy"
 DRIVERS="$DRIVERS drivers/net/wireless/iwlwifi"
 DRIVERS="$DRIVERS drivers/net/wireless/rt2x00"
 DRIVERS="$DRIVERS drivers/net/wireless/zd1211rw"
 DRIVERS="$DRIVERS drivers/net/wireless/libertas"
 DRIVERS="$DRIVERS drivers/net/wireless/p54"
 DRIVERS="$DRIVERS drivers/net/wireless/rtl818x"
+DRIVERS="$DRIVERS drivers/net/wireless/rtl818x/rtl8180"
+DRIVERS="$DRIVERS drivers/net/wireless/rtl818x/rtl8187"
+DRIVERS="$DRIVERS drivers/net/wireless/rtlwifi"
+DRIVERS="$DRIVERS drivers/net/wireless/rtlwifi/rtl8192c"
+DRIVERS="$DRIVERS drivers/net/wireless/rtlwifi/rtl8192ce"
+DRIVERS="$DRIVERS drivers/net/wireless/rtlwifi/rtl8192cu"
+DRIVERS="$DRIVERS drivers/net/wireless/rtlwifi/rtl8192se"
+DRIVERS="$DRIVERS drivers/net/wireless/rtlwifi/rtl8192de"
 DRIVERS="$DRIVERS drivers/net/wireless/libertas_tf"
 DRIVERS="$DRIVERS drivers/net/wireless/ipw2x00"
 DRIVERS="$DRIVERS drivers/net/wireless/wl12xx"
+DRIVERS="$DRIVERS drivers/net/wireless/wl1251"
 DRIVERS="$DRIVERS drivers/net/wireless/iwmc3200wifi"
 DRIVERS="$DRIVERS drivers/net/wireless/orinoco"
+DRIVERS="$DRIVERS drivers/net/wireless/mwifiex"
+
+# Staging drivers
+STAGING_DRIVERS="drivers/staging/ath6kl"
+STAGING_DRIVERS="$STAGING_DRIVERS drivers/staging/brcm80211"
 
 # Ethernet drivers
 DRIVERS="$DRIVERS drivers/net/atl1c"
@@ -244,7 +261,7 @@ DRIVERS_BT="drivers/bluetooth"
 # Drivers that belong the the wireless directory
 DRIVER_FILES="adm8211.c  adm8211.h"
 DRIVER_FILES="$DRIVER_FILES rndis_wlan.c"
-DRIVER_FILES="$DRIVER_FILES mac80211_hwsim.c"
+DRIVER_FILES="$DRIVER_FILES mac80211_hwsim.c mac80211_hwsim.h"
 DRIVER_FILES="$DRIVER_FILES at76c50x-usb.c at76c50x-usb.h"
 DRIVER_FILES="$DRIVER_FILES mwl8k.c"
 
@@ -255,9 +272,11 @@ mkdir -p include/linux/ include/net/ include/linux/usb \
 	include/linux/spi \
 	include/trace \
 	include/pcmcia \
+	include/crypto \
 	net/mac80211/ net/wireless/ \
 	net/rfkill/ \
 	drivers/ssb/ \
+	drivers/bcma/ \
 	drivers/net/usb/ \
 	drivers/net/wireless/
 mkdir -p include/net/bluetooth/
@@ -270,6 +289,7 @@ for i in $INCLUDE_LINUX; do
 done
 
 cp -a $GIT_TREE/include/linux/ssb include/linux/
+cp -a $GIT_TREE/include/linux/bcma include/linux/
 cp -a $GIT_TREE/include/linux/rfkill.h include/linux/rfkill_backport.h
 
 # include/net
@@ -326,6 +346,20 @@ for i in $DRIVERS; do
 	rm -f $i/*.mod.c
 done
 
+# Staging drivers in their own directory
+rm -rf drivers/staging/
+mkdir -p drivers/staging/
+for i in $STAGING_DRIVERS; do
+	if [ ! -d $GIT_TREE/$i ]; then
+		continue
+	fi
+	rm -rf $i
+	echo -e "Copying ${RED}STAGING${NORMAL} $GIT_TREE/$i/*.[ch]"
+	# staging drivers tend to have their own subdirs...
+	cp -a $GIT_TREE/$i drivers/staging/
+	rm -f $i/*.mod.c
+done
+
 for i in $DRIVERS_BT; do
 	mkdir -p $i
 	echo "Copying $GIT_TREE/$i/*.[ch]"
@@ -378,6 +412,7 @@ cp -a $GIT_COMPAT_TREE/include/linux/* include/linux/
 cp -a $GIT_COMPAT_TREE/include/net/* include/net/
 cp -a $GIT_COMPAT_TREE/include/trace/* include/trace/
 cp -a $GIT_COMPAT_TREE/include/pcmcia/* include/pcmcia/
+cp -a $GIT_COMPAT_TREE/include/crypto/* include/crypto/
 rm -f $COMPAT/*.mod.c
 
 # files we suck in for compat-wireless
@@ -401,6 +436,23 @@ export WSTABLE="
 
 # Stable pending, if -n was passed
 if [[ "$GET_STABLE_PENDING" = y ]]; then
+
+	if [ -z $NEXT_TREE ]; then
+		NEXT_TREE="/home/$USER/linux-next/"
+		if [ ! -d $NEXT_TREE ]; then
+			echo "Please tell me where your linux-next git tree is."
+			echo "You can do this by exporting its location as follows:"
+			echo
+			echo "  export NEXT_TREE=/home/$USER/linux-next/"
+			echo
+			echo "If you do not have one you can clone the repository:"
+			echo "  git clone git://git.kernel.org/pub/scm/linux/kernel/git/next/linux-next.git"
+			exit 1
+		fi
+	else
+		echo "You said to use git tree at: $NEXT_TREE for linux-next"
+	fi
+
 	LAST_DIR=$PWD
 	cd $GIT_TREE
 	if [ -f localversion* ]; then
@@ -411,16 +463,19 @@ if [[ "$GET_STABLE_PENDING" = y ]]; then
 	# we now assume you are using a stable tree
 	cd $GIT_TREE
 	LAST_STABLE_UPDATE=$(git describe --abbrev=0)
-	NEXT_TREE="/home/$USER/linux-next/"
-	if [ ! -d $NEXT_TREE ]; then
-		echo -e "You are expected to have $NEXT_TREE directory when using -s"
-		exit 1
-	fi
 	cd $NEXT_TREE
 	PENDING_STABLE_DIR="pending-stable/"
 
-	rm -rf $CHERRY_PICK_DIR
+	rm -rf $PENDING_STABLE_DIR
+
+	git tag -l | grep $LAST_STABLE_UPDATE 2>&1 > /dev/null
+	if [[ $? -ne 0 ]]; then
+		echo -e "${BLUE}Tag $LAST_STABLE_UPDATE not found on $NEXT_TREE tree: bailing out${NORMAL}"
+		exit 1
+	fi
 	echo -e "${GREEN}Generating stable cherry picks... ${NORMAL}"
+	echo -e "\nUsing command on directory $PWD:"
+	echo -e "\ngit format-patch --grep=\"stable@kernel.org\" -o $PENDING_STABLE_DIR ${LAST_STABLE_UPDATE}.. $WSTABLE"
 	git format-patch --grep="stable@kernel.org" -o $PENDING_STABLE_DIR ${LAST_STABLE_UPDATE}.. $WSTABLE
 	if [ ! -d ${LAST_DIR}/${PENDING_STABLE_DIR} ]; then
 		echo -e "Assumption that ${LAST_DIR}/${PENDING_STABLE_DIR} directory exists failed"
@@ -486,11 +541,15 @@ for dir in $EXTRA_PATCHES; do
 done
 
 for dir in $EXTRA_PATCHES; do
+	if [[ ! -d $dir ]]; then
+		echo -e "${RED}Patches: $dir empty, skipping...${NORMAL}\n"
+		continue
+	fi
 	if [[ $LAST_ELEM = $dir && "$REFRESH" = y ]]; then
 		patchRefresh $dir
 	fi
 
-	FOUND=$(find $dir/ -name \*.patch | wc -l)
+	FOUND=$(find $dir/ -maxdepth 1 -name \*.patch | wc -l)
 	if [ $FOUND -eq 0 ]; then
 		continue
 	fi
